/* row.cpp
 *
 * Pieter Eendebak ( pte@ddsw.nl )
 *
 */

#include "includes.h"
#include "defines.h"

#include "kmaster.h"

#include "row.moc"


/* ##########################
 * #	class MasterRow           #
 * ##########################
 */

MasterRow::MasterRow ( bool _pixmaps, QPixmap **_pins, const char * _base,
			QWidget *parent, const char *name,
			int _id, int rows, const int *code)
	: QWidget (parent, name)
{
	setMinimumSize(120,20);
	level=rows;
	refcode = code;
	id=_id;
	
	base.sprintf("%s", _base );
	pixmaps=_pixmaps;

	QString p;
	p.setNum(id);
	veld = new QPushButton ( p, this, "veld" );
	connect( veld, SIGNAL(clicked()), this, SLOT(evaluate()) );

	result = new QLabel("-", this);

	knoppen = new Knop *[level];
	for(int i=0;i<level;i++)
		 knoppen[i] = new Knop(pixmaps, _pins, base.data(), this);
	repaint();
}

MasterRow::~MasterRow()
{
	delete veld;
	delete result;
	delete [] knoppen;
}

void MasterRow::resizeEvent( QResizeEvent *)
{
	QSize o = size();

	// New resizing code
	int b = o.width();
	int h = o.height();

	if ( pixmaps )
	{
		int size = 30;
			
		int scaling = size;
		
		veld->setGeometry( 10, 0, size, size );
		for(int i=0;i<level;i++)
			knoppen[i]->setGeometry(20+(1+i)*scaling, 0,
					size, size );

		result->move( 30 + ((level+1)*size) , 1 );
	}
	else
	{	
		// 10: extra space for veld; 35: size result
		int vsize=h-2;
		int hsize = (b - 35 - 10)/(level+1);

		int size = hsize;
		if  ( size > vsize ) size=vsize;
	
		// if (hsize-30>vsize);
		int widgetsize = (int)( (size*4)/5 );
	
		veld->setGeometry(0, 2, widgetsize, widgetsize );
		for(int i=0;i<level;i++)
			knoppen[i]->setGeometry(10+(1+i)*size, 2,
					widgetsize, widgetsize );

		result->move( 10 + ((level+1)*size) , 2 );
	}
}


void MasterRow::paintEvent( QPaintEvent * )
{
	for(int i=0;i<level;i++) knoppen[i]->repaint();
}

void MasterRow::setPixMap(bool yesno)
{
	pixmaps=yesno;
	for(int i=0;i<level;i++)
	{
		knoppen[i]->pixmaps=yesno;
	}

	repaint();
}


void MasterRow::setState(int s)
{
	// state : 0=editable 1=not editable else=future functions

	status=s;
	
	if (s==0)
	{
		for(int i=0;i<level;i++) knoppen[i]->editable=true;
	}
	else if (s==1)
	{
		for(int i=0;i<level;i++) knoppen[i]->editable=false;
	}

	repaint();
}

int MasterRow::getState()
{
	return status;
}

void MasterRow::evaluate()
{
	int good=0, place=0;
	int array[2][7];

	for(int i=0;i<7;i++) array[0][i]=0,array[1][i]=0;

	// check if the user has clicked all buttons...
	for(int i=0;i<level;i++) if (knoppen[i]->readColor()==0) return;
	
	// has many colours has the users on the right place?
	for(int i=0;i<level;i++)
	if (refcode[i]==knoppen[i]->readColor() ) good++;
	
	for(int i=0;i<level;i++) array[0][ refcode[i] ]++;
	for(int i=0;i<level;i++) array[1][ knoppen[i]->readColor() ]++;

	for(int i=1;i<7;i++)
	{
		if (array[1][i]!=0)
		{
			if(array[0][i]>=array[1][i]) place+=array[1][i];
			if(array[0][i]<array[1][i]) place+=array[0][i];
		}
	}

	place-=good;
	
	// we have calculated the results
	// now we decide what to do

	setState(0);	// make sure the row cannot be edited any more

	// show results
	char s[100];
	sprintf(s, "%d, %d", good,place);
	result->setText(s);
	
	if (good==level)
	{
		emit changed(id, 1);
		result->setText( klocale->translate("Good!") );
	}
	else
	{
		emit changed(id,0);
	}
	repaint();
}


/* ##########################
 * #	class Knop          #
 * ##########################
 */

void Knop::resizeEvent( QResizeEvent * )
{
	s = size();
	hoog=s.height();
	breed=s.width();
	
	if ( pixmaps )
	{
		// code
	}
	else
	{
		plaat = new QPixmap( hoog, breed);
		repaint();
	}
}

Knop::Knop (bool _pixmaps, QPixmap ** _pins, const char *_base,
		QWidget *parent, const char *name )
		: QWidget (parent, name)
{
	setMinimumSize(15,15);
	setMaximumSize(70,70);

	base.sprintf("%s", _base);
	pixmaps=_pixmaps;	
	pins=_pins;

	plaat = new QPixmap();
	kleur = 0;
	editable=false;

	br = new QBrush();	
	pt = new QPainter();
}

Knop::~Knop()
{
	delete br;
	delete pt;
	delete plaat;
}

void Knop::mouseReleaseEvent( QMouseEvent *mouse )
{
	if (mouse->button()==LeftButton) changecolor(1);
	else if (mouse->button()==RightButton) changecolor(-1);
}

void Knop::changecolor(int ch)
{
	if (editable)
	{
		kleur+=ch;
		if (kleur>6) kleur=1;
		if (kleur<1) kleur=6;
	}
	this->repaint();
}

void Knop::paintEvent(QPaintEvent *)
{
	if ( pixmaps )
	{
		plaat->resize(hoog,breed);

		bitBlt(plaat, 0, 0, pins[kleur] );
		bitBlt (this, 0, 0, plaat);
	}
	else
	{
		switch(kleur)
		{
		case 0 : br->setColor( backgroundColor() ); break;
		case 1 : br->setColor(black); break;
		case 2 : br->setColor(red); break;
		case 3 : br->setColor(yellow); break;
		case 4 : br->setColor(white); break;
		case 5 : br->setColor(green); break;
		case 6 : br->setColor(blue); break;
		};

		br->setStyle(SolidPattern);
 
		plaat->fill( backgroundColor() );
		pt->begin(plaat);
		pt->setPen(black);
		pt->drawRect(0,0, breed, hoog );
		pt->fillRect(1,1, breed-2, hoog-2,*br);
		pt->end();

		bitBlt (this, 0, 0, plaat);
	}
}

void Knop::setColor(int k)
{
	kleur = k ;
}

int Knop::readColor()
{
	return kleur;
}



